@@ -1,4 +1,4 @@
-# Copyright (C) 2014-2017 New York University
+# Copyright (C) 2014 New York University
 # This file is part of ReproZip which is released under the Revised BSD License
 # See file LICENSE for full license details.
 
@@ -18,6 +18,9 @@ from __future__ import division, print_function, unicode_literals
 
 import argparse
 import copy
+from elftools.common.exceptions import ELFError
+from elftools.elf.elffile import ELFFile
+from elftools.elf.segments import InterpSegment
 import logging
 import os
 import platform
@@ -45,6 +48,17 @@ from reprounzip.utils import unicode_, irange, iteritems, itervalues, \
 logger = logging.getLogger('reprounzip')
 
 
+def get_elf_interpreter(file):
+    try:
+        elf = ELFFile(file)
+        for segment in elf.iter_segments():
+            if isinstance(segment, InterpSegment):
+                return segment.get_interp_name()
+        return None
+    except ELFError:
+        return None
+
+
 def installpkgs(args):
     """Installs the necessary packages on the current machine.
     """
@@ -62,6 +76,7 @@ def installpkgs(args):
         installer = select_installer(pack, runs)
     except CantFindInstaller as e:
         logger.error("Couldn't select a package installer: %s", e)
+        sys.exit(1)
 
     if args.summary:
         # Print out a list of packages with their status
@@ -203,30 +218,38 @@ def directory_run(args):
 
     # Gets library paths
     lib_dirs = []
+    logger.debug("Running: %s", "/sbin/ldconfig -v -N")
     p = subprocess.Popen(['/sbin/ldconfig', '-v', '-N'],
-                         stdout=subprocess.PIPE)
+                         stdin=subprocess.PIPE,
+                         stdout=subprocess.PIPE,
+                         stderr=subprocess.PIPE)
+    stdout, _ = p.communicate()
     try:
-        for l in p.stdout:
-            if len(l) < 3 or l[0] in (b' ', b'\t'):
+        for line in stdout.splitlines():
+            if len(line) < 2 or line[0] in (b' ', b'\t'):
                 continue
-            if l.endswith(b':\n'):
-                lib_dirs.append(Path(l[:-2]))
+            if line.endswith(b':'):
+                lib_dirs.append(Path(line[:-1]))
     finally:
-        p.wait()
-    lib_dirs = ('export LD_LIBRARY_PATH=%s' % ':'.join(
-                shell_escape(unicode_(join_root(root, d)))
-                for d in lib_dirs))
+        if p.returncode != 0:
+            raise subprocess.CalledProcessError(p.returncode,
+                                                ['/sbin/ldconfig', '-v', '-N'])
 
-    cmds = [lib_dirs]
+    cmds = []
     for run_number in selected_runs:
         run = runs[run_number]
         cmd = 'cd %s && ' % shell_escape(
             unicode_(join_root(root,
                                Path(run['workingdir']))))
         cmd += '/usr/bin/env -i '
+        cmd += 'LD_LIBRARY_PATH=%s ' % ':'.join(
+            shell_escape(unicode_(join_root(root, d)))
+            for d in lib_dirs
+        )
         environ = run['environ']
         environ = fixup_environment(environ, args)
         if args.x11:
+            environ = dict(environ)
             if 'DISPLAY' in os.environ:
                 environ['DISPLAY'] = os.environ['DISPLAY']
             if 'XAUTHORITY' in os.environ:
@@ -248,10 +271,22 @@ def directory_run(args):
         path = ':'.join(unicode_(d) for d in dir_path + path)
         cmd += 'PATH=%s ' % shell_escape(path)
 
+        interpreter = get_elf_interpreter(
+            join_root(root, PosixPath(run['binary'])).open('rb'),
+        )
+        if interpreter is not None:
+            interpreter = Path(interpreter)
+            if interpreter.exists():
+                cmd += '%s ' % shell_escape(str(join_root(root, interpreter)))
+
         # FIXME : Use exec -a or something if binary != argv[0]
         if cmdline is None:
             argv = run['argv']
 
+            # If the command is not a path, use the path instead
+            if '/' not in argv[0]:
+                argv = [run['binary']] + argv[1:]
+
             # Rewrites command-line arguments that are absolute filenames
             rewritten = False
             for i in irange(len(argv)):
@@ -275,6 +310,7 @@ def directory_run(args):
     cmds = ' && '.join(cmds)
 
     signals.pre_run(target=target)
+    logger.debug("Running: %s", cmds)
     retcode = interruptible_call(cmds, shell=True)
     stderr.write("\n*** Command finished, status: %d\n" % retcode)
     signals.post_run(target=target, retcode=retcode)
@@ -576,6 +612,7 @@ def chroot_run(args):
         forwarders.append(fwd)
 
     signals.pre_run(target=target)
+    logger.debug("Running: %s", cmds)
     retcode = interruptible_call(cmds, shell=True)
     stderr.write("\n*** Command finished, status: %d\n" % retcode)
     signals.post_run(target=target, retcode=retcode)
